package com.tanpu.community.service.quartz;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.tanpu.common.api.CommonResp;
import com.tanpu.common.constant.BizStatus;
import com.tanpu.common.exception.BizException;
import com.tanpu.common.util.DateUtils;
import com.tanpu.community.api.beans.vo.feign.fatools.SimpleUserInfo;
import com.tanpu.community.dao.entity.community.TopicEntity;
import com.tanpu.community.dao.entity.community.TopicManagerEntity;
import com.tanpu.community.dao.entity.community.WxCpUser;
import com.tanpu.community.dao.mapper.community.WxCpUserMapper;
import com.tanpu.community.feign.fatools.FeignClientForFatools;
import com.tanpu.community.feign.wxcp.FeignClientForWxcp;
import com.tanpu.community.service.CommentService;
import com.tanpu.community.service.ThemeService;
import com.tanpu.community.service.TopicService;
import com.tanpu.community.service.VisitLogService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class TopicReportService {

    @Value("${wxcp.topicreport.agentId}")
    private Long agentId;

    @Value("${wxcp.topicreport.corpId}")
    private String corpId;

    @Autowired
    private TopicService topicService;

    @Autowired
    private ThemeService themeService;

    @Autowired
    private CommentService commentService;

    @Autowired
    private VisitLogService visitLogService;

    @Autowired
    private FeignClientForWxcp feignClientForWxcp;

    @Autowired
    private FeignClientForFatools feignClientForFatools;

    @Resource
    private WxCpUserMapper wxCpUserMapper;


    public static String content_report_topic_ontime = "话题(%s) 有一个新的讨论/评论";
    public void reportTopicOnTime(String topicId) {
        TopicEntity topic = topicService.queryById(topicId);
        if (topic == null || topic.getSyncCorpWechat() == 0) return;

        String content = String.format(content_report_topic_ontime, topic.getTopicTitle());
        List<String> wxcpIds = getTopicManagerWxCpIds(topic.getTopicId());
        sendReportMessage(content, wxcpIds);
    }


    // 每个工作日早上09:30汇报
    public static String content_report_topic_weekday = "话题(%s) 当前总成员: %s, 当前总阅读: %s, 当前总讨论: %s, 昨日新增成员: %s, 昨日新增讨论: %s";
    public void reportTopicWeekday() {
        // 跳过双休日
        Date now = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(now);
        int w = cal.get(Calendar.DAY_OF_WEEK) - 1;
        if (w == 6 || w == 7) return;

        List<TopicEntity> topics = topicService.queryTopicNeedReport();
        for (TopicEntity topic : topics) {
            log.info("reportTopicWeekday for topic: " + topic.getTopicId());

            // 当前总成员
            Integer totalFollowNum = topicService.countTotalFollow(topic.getTopicId());

            // 阅读量
            Integer totalVisitNum = visitLogService.queryTopicDetailVisit(topic.getTopicId());

            // 讨论量 包括主题数量和评论数量 不算长文
            List<String> totalThemeIds = themeService.queryThemeIdsByTopic(topic.getTopicId());
            if (totalThemeIds.isEmpty()) continue;
            Integer totalCommentNum = commentService.getCommentCountByThemeIds(totalThemeIds);
            Integer totalDiscussNum = totalCommentNum + totalThemeIds.size();

            Date endDate = DateUtils.getDayStartTime(new Date());
            Date startDate = DateUtils.getAfterDay(endDate, -1);
            // 昨日新增成员
            Integer newFollowNum = topicService.countFollowByIdAndTime(topic.getTopicId(), startDate, endDate);

            // 昨日新增讨论
            List<String> newThemeIds = themeService.queryThemeIdsByTopic(topic.getTopicId(), startDate, endDate);
            Integer newCommentNum = commentService.getCommentCountByThemeIds(totalThemeIds, startDate, endDate);
            Integer newDiscussNum = newThemeIds.size() + newCommentNum;

            String content = String.format(content_report_topic_weekday, topic.getTopicTitle(), totalFollowNum, totalVisitNum, totalDiscussNum, newFollowNum, newDiscussNum);
            List<String> wxcpIds = getTopicManagerWxCpIds(topic.getTopicId());
            sendReportMessage(content, wxcpIds);
        }
    }

    public static void main(String[] args) {
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int w = cal.get(Calendar.DAY_OF_WEEK) - 1;

        System.out.println(w);
    }

    // 每周六早上09:00汇报
    public static String content_report_topic_saturday = "话题(%s) 当前总成员: %s, 当前总阅读: %s, 当前总讨论: %s, 截至本周五新增成员: %s, 昨日新增讨论: %s";
    public void reportTopicSaturday() {
        List<TopicEntity> topics = topicService.queryTopicNeedReport();
        for (TopicEntity topic : topics) {
            log.info("reportTopicSaturday for topic: " + topic.getTopicId());

            // 当前总成员
            Integer totalFollowNum = topicService.countTotalFollow(topic.getTopicId());

            // 阅读量
            Integer totalVisitNum = visitLogService.queryTopicDetailVisit(topic.getTopicId());

            // 讨论量 包括主题数量和评论数量 不算长文
            List<String> totalThemeIds = themeService.queryThemeIdsByTopic(topic.getTopicId());
            Integer totalCommentNum = commentService.getCommentCountByThemeIds(totalThemeIds);
            Integer totalDiscussNum = totalCommentNum + totalThemeIds.size();


            Date today = DateUtils.getDayStartTime(new Date());
            Date endDate = DateUtils.getAfterDay(today, -1);
            Date startDate = DateUtils.getAfterDay(endDate, -5);
            // 昨日新增成员
            Integer newFollowNum = topicService.countFollowByIdAndTime(topic.getTopicId(), startDate, endDate);

            // 昨日新增讨论
            List<String> newThemeIds = themeService.queryThemeIdsByTopic(topic.getTopicId(), startDate, endDate);
            if (newThemeIds.isEmpty()) continue;
            Integer newCommentNum = commentService.getCommentCountByThemeIds(totalThemeIds, startDate, endDate);
            Integer newDiscussNum = newThemeIds.size() + newCommentNum;

            String content = String.format(content_report_topic_saturday, topic.getTopicTitle(), totalFollowNum, totalVisitNum, totalDiscussNum, newFollowNum, newDiscussNum);
            List<String> wxcpIds = getTopicManagerWxCpIds(topic.getTopicId());
            sendReportMessage(content, wxcpIds);
        }
    }


    public void sendReportMessage(String content, List<String> wxcpIds) {
        log.info("sendTopicReportMessage wxcpIds: {}, content: {}", JSON.toJSONString(wxcpIds), content);
        if (CollectionUtils.isEmpty(wxcpIds)) {
            return;
        }

        Map<String, Object> parmMap = new HashMap<>();
        parmMap.put("type", "text");
        parmMap.put("content", content);
        parmMap.put("list", wxcpIds);

        try {
            feignClientForWxcp.sendMessage(agentId, corpId, JSON.toJSONString(parmMap));
        }catch (Exception e){
            log.error("圈子报告企业微信，发送wxmp微信客服消息失败，error：{}",e.getMessage());
        }
    }

    public List<String> getTopicManagerWxCpIds(String topicId) {
        List<TopicManagerEntity> managers = topicService.getManagersByTopic(topicId);

        List<String> userIdList = managers.stream().map(TopicManagerEntity::getUserId).collect(Collectors.toList());
        CommonResp<List<SimpleUserInfo>> resp = feignClientForFatools.simpleUserList(userIdList);
        if (resp.isNotSuccess()) {
            throw new BizException(resp.getCode(), resp.getMsg());
        }
        List<String> mobileList = resp.getData().stream()
                .map(SimpleUserInfo::getUiTelphone)
                .filter(StringUtils::isNotEmpty).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(mobileList)) {
            return Lists.newArrayListWithCapacity(0);
        }
        List<WxCpUser> wxCpUsers = wxCpUserMapper.selectList(
                Wrappers.<WxCpUser>lambdaQuery()
                        .in(WxCpUser::getMobile, mobileList)
                        .eq(WxCpUser::getDeleteTag, BizStatus.DeleteTag.tag_init)
                        .select(WxCpUser::getUserId)
        );
        return wxCpUsers.stream().map(WxCpUser::getUserId).collect(Collectors.toList());
    }
}
